// -*- C++ -*-
// File: init.c
//
// Created: Tue Dec  8 01:18:13 1998
//
// $Id: init_deinit.c,v 1.13 1999/10/06 12:11:32 yarick Exp $
//
#include <gtk/gtk.h>
#include <glib.h>
#include <strings.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>

#include "common.h"
#include "dialogs.h"
#ifndef MAXHOSTNAMELEN
#include <rpc/types.h>
#endif /* MAXHOSTNAMELEN */
#ifdef __DEBUG__
#include "debug.h"
#endif /* __DEBUG__ */
#include "filebrowser.h"
#include "gvfs.h"
#include "history_box.h"
#include "init_deinit.h"
#include "itemfactory.h"
#include "make_dir_list.h"
#include "mount_list.h"
#include "pixmaps.h"
#include "rq.h"
#include "statusbar.h"
#include "toolbar.h"
#include "tree.h"
#include "config.h"

static gchar *config_file, *td_file;

void
init_rc ()
{
  //proto for .rc file parsing func
  config_file = g_malloc0 (PATH_MAX);
  td_file = g_malloc0 (PATH_MAX);
  strcpy (config_file, getenv ("HOME"));
  append_slash (config_file);
  strcpy (td_file, config_file);
  strcat (config_file, CONFIG_FILE);
  strcat (td_file, TD_FILE);
  load_config (config_file);
  load_typedata (td_file);
  g_free (config_file);
  g_free (td_file);
}

void
deinit_rc ()
{
  config_file = g_malloc0 (PATH_MAX);
  td_file = g_malloc0 (PATH_MAX);
  strcpy (config_file, getenv ("HOME"));
  append_slash (config_file);
  strcpy (td_file, config_file);
  strcat (config_file, CONFIG_FILE);
  strcat (td_file, TD_FILE);
  save_config (config_file);
  save_typedata (td_file);
  g_free (config_file);
  g_free (td_file);
}

void
init_commandline()
{
    gchar *hgc1;gint hgi1;
    for (hgi1 = 1; hgi1<argc; hgi1++)
        {
            hgc1 = argv[hgi1];
            if (!strcmp(hgc1, "-t"))
                tree_view = FALSE;
                else
            gvfs_set_current_dir(hgc1);
        };
};

void
create_toolbtns (void)
{
  GtkWidget *pm;
  GtkStyle *style;
  GfbItemIcons *hGII1;
  style = gtk_widget_get_style (main_window);
  hGII1 = GFB_ITEM_ICONS (GFB_TYPE_DEF (find_typedef ("GFB_UPDIR_BUTTON"))->icons);
  pm = gtk_pixmap_new (hGII1->Icon1, hGII1->Bitmap1);
  gtk_widget_show (pm);
  gtk_toolbar_append_space (GTK_TOOLBAR (main_toolbox));
  gtk_toolbar_append_item (GTK_TOOLBAR (main_toolbox), "Up",
		"Up one level", "Private", pm, GTK_SIGNAL_FUNC (toolbar_cb),
			   (void *) TBAR_CDUP);
  gtk_toolbar_append_space (GTK_TOOLBAR (main_toolbox));

  hGII1 = GFB_ITEM_ICONS (GFB_TYPE_DEF (find_typedef ("GFB_CUT_BUTTON"))->icons);
  pm = gtk_pixmap_new (hGII1->Icon1, hGII1->Bitmap1);
  gtk_widget_show (pm);
  gtk_toolbar_append_item (GTK_TOOLBAR (main_toolbox), "Cut",
	"Cut selected file(s)", "Private", pm, GTK_SIGNAL_FUNC (toolbar_cb),
			   (void *) TBAR_CUT);

  hGII1 = GFB_ITEM_ICONS (GFB_TYPE_DEF (find_typedef ("GFB_COPY_BUTTON"))->icons);
  pm = gtk_pixmap_new (hGII1->Icon1, hGII1->Bitmap1);
  gtk_widget_show (pm);
  gtk_toolbar_append_item (GTK_TOOLBAR (main_toolbox), "Copy",
       "Copy selected file(s)", "Private", pm, GTK_SIGNAL_FUNC (toolbar_cb),
			   (void *) TBAR_COPY);

  hGII1 = GFB_ITEM_ICONS (GFB_TYPE_DEF (find_typedef ("GFB_PASTE_BUTTON"))->icons);
  pm = gtk_pixmap_new (hGII1->Icon1, hGII1->Bitmap1);
  gtk_widget_show (pm);
  gtk_toolbar_append_item (GTK_TOOLBAR (main_toolbox), "Paste",
      "Paste selected file(s)", "Private", pm, GTK_SIGNAL_FUNC (toolbar_cb),
			   (void *) TBAR_PASTE);
  gtk_toolbar_append_space (GTK_TOOLBAR (main_toolbox));


  hGII1 = GFB_ITEM_ICONS (GFB_TYPE_DEF (find_typedef ("GFB_DELETE_BUTTON"))->icons);
  pm = gtk_pixmap_new (hGII1->Icon1, hGII1->Bitmap1);
  gtk_widget_show (pm);
  gtk_toolbar_append_item (GTK_TOOLBAR (main_toolbox), "Delete",
     "Delete selected file(s)", "Private", pm, GTK_SIGNAL_FUNC (toolbar_cb),
			   (void *) TBAR_DELETE);

  hGII1 = GFB_ITEM_ICONS (GFB_TYPE_DEF (find_typedef ("GFB_OPTIONS_BUTTON"))->icons);
  pm = gtk_pixmap_new (hGII1->Icon1, hGII1->Bitmap1);
  gtk_widget_show (pm);
  gtk_toolbar_append_item (GTK_TOOLBAR (main_toolbox), "Options",
		     "Options", "Private", pm, GTK_SIGNAL_FUNC (toolbar_cb),
			   (void *) TBAR_OPTS);
}

void
construct_toolbox ()
{
// constructing toolbox (dir and buttons)

  toolbox_hbox = gtk_hbox_new (FALSE, 10);
  gtk_widget_set_usize (GTK_WIDGET (toolbox_hbox), 400, 30);
  main_history_box = history_box_new ();
  gtk_widget_show (main_history_box);
  gtk_box_pack_start (GTK_BOX (toolbox_hbox), main_history_box, TRUE, TRUE, 0);
  main_toolbox = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL,
				  GTK_TOOLBAR_ICONS);
  create_toolbtns ();
  gtk_box_pack_start (GTK_BOX (toolbox_hbox), main_toolbox, TRUE,
		      TRUE, 0);
  gtk_widget_show (main_toolbox);
}

void
init_widgets ()
{
//func for widgets init (size, pos, lastdir, etc. ) 
  GfbItemIcons *hGII1;
  initQueue ();
  main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize (GTK_WIDGET (main_window),
			main_window_usize_x, main_window_usize_y);

  main_window_destroy_handler = gtk_signal_connect (GTK_OBJECT (main_window), "destroy",
					  GTK_SIGNAL_FUNC (gfb_exit), NULL);
  gtk_window_set_title (GTK_WINDOW (main_window), "GTK File Browser");
  gtk_widget_show (main_window);

  load_icons ();
  hGII1 = GFB_TYPE_DEF(find_typedef("GFB_MAIN_ICON"))->icons;
  gdk_window_set_icon(main_window->window, NULL, hGII1->Icon1, hGII1->Bitmap1);
  main_vbox = gtk_vbox_new (FALSE, 0);
  treelist_vbox = gtk_vbox_new (FALSE, 0);

  get_main_menu (main_window, &main_menubar);
  gtk_widget_show (main_menubar);

  main_listwin = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (main_listwin),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  main_list = gtk_hbox_new (TRUE, 0);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (main_listwin), main_list);
  gtk_widget_show (main_list);

  main_treewin = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_usize (main_treewin, main_treewin_usize_x, 1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (main_treewin),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_widget_show (main_treewin);
  main_tree = construct_tree ();
  gtk_widget_show (main_tree);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (main_treewin), main_tree);

  main_hpaned = gtk_hpaned_new ();

  gtk_paned_add1 (GTK_PANED (main_hpaned), main_treewin);
  gtk_widget_show (main_treewin);
  gtk_paned_add2 (GTK_PANED (main_hpaned), main_listwin);
  create_dir_list (main_list);
  gtk_widget_show (main_listwin);

  gtk_widget_show (main_hpaned);

  main_statusbar = gtk_statusbar_new ();
  gtk_widget_show (main_statusbar);
  init_statusbar ();

  gtk_box_pack_start (GTK_BOX (treelist_vbox), main_hpaned, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (treelist_vbox), main_statusbar, FALSE, FALSE, 0);
  construct_toolbox ();
  gtk_box_pack_start (GTK_BOX (main_vbox), main_menubar, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (main_vbox), toolbox_hbox, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (main_vbox), treelist_vbox, TRUE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (main_window), main_vbox);
  gtk_widget_show (treelist_vbox);
  gtk_widget_show (toolbox_hbox);
  gtk_widget_show (main_vbox);
  putRequest (EXPAND_TREE, gvfs_get_current_dir (), NULL);
}

void
init_filesystem (void)
{
  /* Get the local hostname. */
  if ((gethostname (LOCAL_HOSTNAME, MAXHOSTNAMELEN) != 0) &&
      (getdomainname (LOCAL_HOSTNAME, MAXHOSTNAMELEN) != 0))
    strcpy (LOCAL_HOSTNAME, "LocalHost");

  /* Build the local fileystem mount list. */
  if (build_mount_list () == 0)
    exit (1);

  /* Build the SMB share list. */

  /* Initialize the virtual filesystem layer. */
  if (gvfs_initialize () == 0)
    exit (1);
}

void
preset_typedefs_lookup_table ()
{
  GFB_stddefs[TREE_NODE_WORLD] = "TREE_NODE_WORLD";
  GFB_stddefs[TREE_NODE_LOCALHOST] = "TREE_NODE_LOCALHOST";
  GFB_stddefs[TREE_NODE_HOST] = "TREE_NODE_HOST";
  GFB_stddefs[TREE_NODE_NETWORK] = "TREE_NODE_NETWORK";
  GFB_stddefs[TREE_NODE_HOTLIST] = "TREE_NODE_HOTLIST";		/* Hotlist Node        */
  GFB_stddefs[TREE_NODE_HOTLIST_ENTRY] = "TREE_NODE_HOTLIST_ENTRY";	/* Hotlist Entry       */
  GFB_stddefs[TREE_NODE_FILESYSTEM] = "TREE_NODE_FILESYSTEM";	/* Filesystem Node     */
  GFB_stddefs[TREE_NODE_DIR] = "TREE_NODE_DIR";		/* Directory Node      */
  GFB_stddefs[FILE_LIST_DIR] = "FILE_LIST_DIR";		/* FileList Directory  */
  GFB_stddefs[FILE_LIST_FILE] = "FILE_LIST_FILE";	/* FileList File       */
  GFB_stddefs[FILE_LIST_FILESYSTEM] = "FILE_LIST_FILESYSTEM";	/* FileList Filesystem */
}

void
verify_config_dir_exist (void)
{
  struct stat dir_stat;
  int stat_ret;
  gchar *config_dir_path;

  config_dir_path = g_malloc0 (PATH_MAX);
  strcpy (config_dir_path, getenv ("HOME"));
  append_slash (config_dir_path);
  strcat (config_dir_path, ".gfb");
  memset ((void *) &dir_stat, sizeof (struct stat), 0);
  stat_ret = stat (config_dir_path, &dir_stat);
  if (stat_ret == -1)
    mkdir (config_dir_path, 0777);
  g_free (config_dir_path);
}


void
gfb_init (void)
{
  preset_typedefs_lookup_table ();
  /* Patch by Ian Baird allowing the clipboard to init */
  verify_config_dir_exist ();
  /* Initialize the filesystem data. */
  init_filesystem ();
  /* Load and initialize resources and configuration. */
  init_rc ();
  init_commandline();
  /* Create GUI objects and hook up events. */
  init_clipboard ();
  init_widgets ();
}

void
gfb_exit (void)
{
  gtk_signal_disconnect (GTK_OBJECT (main_window), main_window_destroy_handler);
  deInitQueue ();
  shut_clipboard ();
  deinit_rc ();
  debug_off ();
  gvfs_shutdown ();
  gtk_exit (0);
}








